#!/usr/bin/perl
#
# abstractart3.cgi:	produce a single mondrian style image of random lines
#			and filled boxes, avoiding overlaps by a new mechanism
#			and showing all the stages
#
#	(C) Duncan C. White, 23rd February 1999
#
use strict;
use GD;
use webprompt;
use col;


sub min ($$) { return $_[ $_[0] > $_[1] ]; }
sub max ($$) { return $_[ $_[0] < $_[1] ]; }
sub intrand ($) { return $_[0]<1?0:int(rand($_[0])); }


&webprompt::get(
	"abstractart3", "Mondrian-style Abstract Art",
	\&giveback, \&validate, \&createimg,
	qw(X Y Palette=RBY|RBYB|RBYBB|green MinLines MaxLines MinBoxes MaxBoxes MinThickness MaxThickness MinSeparation MaxSeparation) );

exit 0;


sub giveback ($$$$$$$$$$)
{
	my( $xr, $yr, $palette, $minlines, $maxlines, $minboxes,
	    $maxboxes, $minthick, $maxthick, $minsep, $maxsep ) = @_;

	$p'XR = $xr;
	$p'YR = $yr;
	$p'palette = $palette;
	$p'minlines = $minlines;
	$p'maxlines = $maxlines;
	$p'minboxes = $minboxes;
	$p'maxboxes = $maxboxes;
	$p'minthick = $minthick;
	$p'maxthick = $maxthick;
	$p'minsep = $minsep;
	$p'maxsep = $maxsep;

	return @_;
}


sub validate ()
{
	return "" if $p'XR > 50 && $p'YR > 50 &&
		$p'minboxes > 0 && $p'maxboxes >= $p'minboxes &&
		$p'minlines > 0 && $p'maxlines >= $p'minlines &&
		$p'minsep > 0 && $p'maxsep >= $p'minsep &&
		$p'minthick > 0 && $p'maxthick >= $p'minthick;
	return "These values are not sensible";
}


sub createimg ()
{
	$p'im = new GD::Image( $p'XR, $p'YR );

	col::read( $p'im, $p'palette );

	$p'im->setStyle($col'blue, $col'blue,
		GD'gdTransparent(), GD'gdTransparent() );

	#$p'im->transparent($col'white);
	$p'im->interlaced('true');

	image();

	return $p'im;
}


sub image ()
{
	my $nlines = $p'minlines + intrand(1+$p'maxlines-$p'minlines);

	#print "Let's try $nlines lines in each direction\n";

	#print "Placing horizontal lines.. ";
	@pic'hlines = placelines( $nlines, $p'YR,
		$p'minsep, $p'maxsep, $p'minthick, $p'maxthick );

	#print "Placing vertical lines.. ";
	@pic'vlines = placelines( $nlines, $p'XR,
		$p'minsep, $p'maxsep, $p'minthick, $p'maxthick );

	@pic'drawlist = ();				# list of boxes to draw

	my $boxes = $p'minboxes + intrand(1+$p'maxboxes-$p'minboxes);

	#print "\nNow let's try $boxes boxes\n\n";

	for( my $i = 0; $i < $boxes; $i++ )
	{
		#print "Trying box $i.. ";
		onebox();
	}

	redraw();
}


sub placelines ($$$$$$)
{
	my( $nlines, $N, $minsep, $maxsep, $minthick, $maxthick ) = @_;

	my @lines = ();
	my @used = ( 0 ) x $N;
	my $nfree = $N;

	for( my $i = 0; $i < $nlines && $nfree > 0; $i++ )
	{
		my $thick = $minthick + intrand(1+$maxthick-$minthick);
		my $sep = $minsep + intrand(1+$maxsep-$minsep);

		my $x = 1 + intrand( $nfree );

		#print "Line $i: chosen thick = $thick, x = $x, sep = $sep, nfree = $nfree\n";
		my $pos;

		for( $pos = 0; $x > 0; $pos++ )
		{
			$x-- unless $used[$pos];
		}
		#print "\tchosen pos = $pos, used = @used\n";

		push( @lines, [ $pos, $thick ] );

		#
		# mark 1+pos-sep..pos+thick+sep as used.
		#
		my $lower = max( 1 + $pos - $sep, 0 );
		my $upper = min( $pos + $thick + $sep, $N-1 );

		#print "\tmarking $lower..$upper as used\n";

		for( $pos = $lower; $pos <= $upper; $pos++ )
		{
			unless( $used[$pos] )
			{
				$used[$pos] = 1;
				$nfree--;
			}
		}
		#print "\tnow, nfree = $nfree, used = @used\n";
	}

	@lines = sort {$$a[0] <=> $$b[0]} @lines;

	if( $lines[0][0] > 0 )
	{
		unshift( @lines, [ 0, 0 ] );
	}
	if( $lines[@lines-1][0] < $N-1 )
	{
		push( @lines, [ $N-1, 0 ] );
	}

	#print scalar(@lines)." at ";
	#my( $l, $pos, $thick );
	#foreach $l (@lines)
	#{
	#	( $pos, $thick ) = @$l;
	#	print $pos." ";
	#}
	#print "\n";

	return @lines;
}


sub onebox ()
{
	my $col = $col'colour[intrand(@col'colour)];

	my $h = intrand( @pic'hlines - 2 );
	my $v = intrand( @pic'vlines - 2 );

	my ( $hw, $vw );

	if( rand() < 0.1 )
	{
		$hw = min( $h + 1 + intrand( @pic'hlines/2 ), @pic'hlines - 1 );
		$vw = min( $v + 1 + intrand( @pic'vlines/2 ), @pic'vlines - 1 );
	} else
	{
		$hw = $h + 1;
		$vw = $v + 1;
	}
	# print "chosen h=$h, v=$v, hw=$hw, vw=$vw, col = $col\n";
	for( my $a = $h; $a < $hw; $a++ )
	{
		for( my $b = $v; $b < $vw; $b++ )
		{
			push( @pic'drawlist, [ $a, $b, $col ] );
		}
	}
}


sub redraw ()
{
	$p'im->filledRectangle( 0, 0, $p'XR-1, $p'YR-1, $col'white );

	foreach my $l (@pic'hlines)
	{
		my( $y, $col, $thick );

		( $y, $thick ) = @$l;

		if( $thick == 0 )
		{
			next;
			#$col = GD'gdStyled();
			#$thick = 1;
		} else
		{
			$col = $col'black;
		}

		for( my $i = 0; $i < $thick; $i++ )
		{
			$p'im->line( 0, $y+$i, $p'XR-1, $y+$i, $col );
		}
	}

	foreach my $l (@pic'vlines)
	{
		my( $x, $col, $thick );

		( $x, $thick ) = @$l;

		if( $thick == 0 )
		{
			next;
			#$col = GD'gdStyled();
			#$thick = 1;
		} else
		{
			$col = $col'black;
		}

		for( my $i = 0; $i < $thick; $i++ )
		{
			$p'im->line( $x+$i, 0, $x+$i, $p'YR, $col );
		}
	}

	foreach my $r (@pic'drawlist)
	{
		my( $h, $v, $col, $thick, $x1, $y1, $x2, $y2 );

		( $h, $v, $col ) = @$r;

		# print "redraw: h=$h, v=$v, col=$col\n";

		( $x1, $thick ) = @{$pic'vlines[$v]}; $x1 += $thick;

		( $x2, $thick ) = @{$pic'vlines[$v+1]}; $x2--;

		( $y1, $thick ) = @{$pic'hlines[$h]}; $y1 += $thick;

		( $y2, $thick ) = @{$pic'hlines[$h+1]}; $y2--;

		$p'im->filledRectangle( $x1, $y1, $x2, $y2, $col );
	}
}
